import type { OpenId4VciCreateCredentialResponseOptions, OpenId4VciCreateCredentialOfferOptions, OpenId4VciCreateIssuerOptions, OpenId4VcIssuerMetadata } from './OpenId4VcIssuerServiceOptions';
import type { AgentContext } from '@credo-ts/core';
import { JwsService, W3cCredentialService } from '@credo-ts/core';
import { OpenId4VcIssuerModuleConfig } from './OpenId4VcIssuerModuleConfig';
import { OpenId4VcIssuerRepository, OpenId4VcIssuerRecord } from './repository';
/**
 * @internal
 */
export declare class OpenId4VcIssuerService {
    private w3cCredentialService;
    private jwsService;
    private openId4VcIssuerConfig;
    private openId4VcIssuerRepository;
    constructor(w3cCredentialService: W3cCredentialService, jwsService: JwsService, openId4VcIssuerConfig: OpenId4VcIssuerModuleConfig, openId4VcIssuerRepository: OpenId4VcIssuerRepository);
    getIssuerMetadata(agentContext: AgentContext, issuerRecord: OpenId4VcIssuerRecord): OpenId4VcIssuerMetadata;
    createCredentialOffer(agentContext: AgentContext, options: OpenId4VciCreateCredentialOfferOptions & {
        issuer: OpenId4VcIssuerRecord;
    }): Promise<{
        credentialOfferPayload: import("@sphereon/oid4vci-common").CredentialOfferPayloadV1_0_11;
        credentialOffer: string;
    }>;
    getCredentialOfferFromUri(agentContext: AgentContext, uri: string): Promise<import("@sphereon/oid4vci-common").CredentialOfferPayloadV1_0_11>;
    createCredentialResponse(agentContext: AgentContext, options: OpenId4VciCreateCredentialResponseOptions & {
        issuer: OpenId4VcIssuerRecord;
    }): Promise<import("@sphereon/oid4vci-common").CredentialResponse>;
    getAllIssuers(agentContext: AgentContext): Promise<OpenId4VcIssuerRecord[]>;
    getByIssuerId(agentContext: AgentContext, issuerId: string): Promise<OpenId4VcIssuerRecord>;
    updateIssuer(agentContext: AgentContext, issuer: OpenId4VcIssuerRecord): Promise<void>;
    createIssuer(agentContext: AgentContext, options: OpenId4VciCreateIssuerOptions): Promise<OpenId4VcIssuerRecord>;
    rotateAccessTokenSigningKey(agentContext: AgentContext, issuer: OpenId4VcIssuerRecord): Promise<void>;
    private getCredentialOfferSessionFromUri;
    private getJwtVerifyCallback;
    private getVcIssuer;
    private getGrantsFromConfig;
    private findOfferedCredentialsMatchingRequest;
    private getSdJwtVcCredentialSigningCallback;
    private getW3cCredentialSigningCallback;
    private getHolderBindingFromRequest;
    private getCredentialDataSupplier;
}
