"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenId4VcSiopVerifierService = void 0;
const core_1 = require("@credo-ts/core");
const did_auth_siop_1 = require("@sphereon/did-auth-siop");
const router_1 = require("../shared/router");
const transform_1 = require("../shared/transform");
const utils_1 = require("../shared/utils");
const OpenId4VcVerifierModuleConfig_1 = require("./OpenId4VcVerifierModuleConfig");
const repository_1 = require("./repository");
/**
 * @internal
 */
let OpenId4VcSiopVerifierService = class OpenId4VcSiopVerifierService {
    constructor(logger, w3cCredentialService, openId4VcVerifierRepository, config) {
        this.logger = logger;
        this.w3cCredentialService = w3cCredentialService;
        this.openId4VcVerifierRepository = openId4VcVerifierRepository;
        this.config = config;
    }
    async createAuthorizationRequest(agentContext, options) {
        var _a;
        const nonce = await agentContext.wallet.generateNonce();
        const state = await agentContext.wallet.generateNonce();
        const correlationId = core_1.utils.uuid();
        const relyingParty = await this.getRelyingParty(agentContext, options.verifier, {
            presentationDefinition: (_a = options.presentationExchange) === null || _a === void 0 ? void 0 : _a.definition,
            requestSigner: options.requestSigner,
        });
        const authorizationRequest = await relyingParty.createAuthorizationRequest({
            correlationId,
            nonce,
            state,
        });
        const authorizationRequestUri = await authorizationRequest.uri();
        return {
            authorizationRequestUri: authorizationRequestUri.encodedUri,
            authorizationRequestPayload: authorizationRequest.payload,
        };
    }
    async verifyAuthorizationResponse(agentContext, options) {
        var _a, _b, _c, _d;
        const authorizationResponse = await did_auth_siop_1.AuthorizationResponse.fromPayload(options.authorizationResponse).catch(() => {
            throw new core_1.CredoError(`Unable to parse authorization response payload. ${JSON.stringify(options.authorizationResponse)}`);
        });
        const responseNonce = await authorizationResponse.getMergedProperty('nonce', {
            hasher: core_1.Hasher.hash,
        });
        const responseState = await authorizationResponse.getMergedProperty('state', {
            hasher: core_1.Hasher.hash,
        });
        const sessionManager = this.config.getSessionManager(agentContext);
        const correlationId = responseNonce
            ? await sessionManager.getCorrelationIdByNonce(responseNonce, false)
            : responseState
                ? await sessionManager.getCorrelationIdByState(responseState, false)
                : undefined;
        if (!correlationId) {
            throw new core_1.CredoError(`Unable to find correlationId for nonce '${responseNonce}' or state '${responseState}'`);
        }
        const requestSessionState = await sessionManager.getRequestStateByCorrelationId(correlationId);
        if (!requestSessionState) {
            throw new core_1.CredoError(`Unable to find request state for correlationId '${correlationId}'`);
        }
        const requestClientId = await requestSessionState.request.getMergedProperty('client_id');
        const requestNonce = await requestSessionState.request.getMergedProperty('nonce');
        const requestState = await requestSessionState.request.getMergedProperty('state');
        const presentationDefinitionsWithLocation = await requestSessionState.request.getPresentationDefinitions();
        if (!requestNonce || !requestClientId || !requestState) {
            throw new core_1.CredoError(`Unable to find nonce, state, or client_id in authorization request for correlationId '${correlationId}'`);
        }
        const relyingParty = await this.getRelyingParty(agentContext, options.verifier, {
            presentationDefinition: (_a = presentationDefinitionsWithLocation === null || presentationDefinitionsWithLocation === void 0 ? void 0 : presentationDefinitionsWithLocation[0]) === null || _a === void 0 ? void 0 : _a.definition,
            clientId: requestClientId,
        });
        const response = await relyingParty.verifyAuthorizationResponse(authorizationResponse.payload, {
            audience: requestClientId,
            correlationId,
            state: requestState,
            presentationDefinitions: presentationDefinitionsWithLocation,
            verification: {
                presentationVerificationCallback: this.getPresentationVerificationCallback(agentContext, {
                    nonce: requestNonce,
                    audience: requestClientId,
                }),
                // FIXME: Supplied mode is not implemented.
                // See https://github.com/Sphereon-Opensource/SIOP-OID4VP/issues/55
                mode: did_auth_siop_1.VerificationMode.INTERNAL,
                resolveOpts: { noUniversalResolverFallback: true, resolver: (0, utils_1.getSphereonDidResolver)(agentContext) },
            },
        });
        const presentationExchange = ((_b = response.oid4vpSubmission) === null || _b === void 0 ? void 0 : _b.submissionData)
            ? {
                submission: response.oid4vpSubmission.submissionData,
                definition: (_c = response.oid4vpSubmission.presentationDefinitions[0]) === null || _c === void 0 ? void 0 : _c.definition,
                presentations: (_d = response.oid4vpSubmission) === null || _d === void 0 ? void 0 : _d.presentations.map(transform_1.getVerifiablePresentationFromSphereonWrapped),
            }
            : undefined;
        const idToken = response.authorizationResponse.idToken
            ? {
                payload: await response.authorizationResponse.idToken.payload(),
            }
            : undefined;
        // TODO: do we need to verify whether idToken or vpToken is present?
        // Or is that properly handled by sphereon's library?
        return {
            // Parameters related to ID Token.
            idToken,
            // Parameters related to DIF Presentation Exchange
            presentationExchange,
        };
    }
    async getAllVerifiers(agentContext) {
        return this.openId4VcVerifierRepository.getAll(agentContext);
    }
    async getByVerifierId(agentContext, verifierId) {
        return this.openId4VcVerifierRepository.getByVerifierId(agentContext, verifierId);
    }
    async updateVerifier(agentContext, verifier) {
        return this.openId4VcVerifierRepository.update(agentContext, verifier);
    }
    async createVerifier(agentContext, options) {
        var _a;
        const openId4VcVerifier = new repository_1.OpenId4VcVerifierRecord({
            verifierId: (_a = options === null || options === void 0 ? void 0 : options.verifierId) !== null && _a !== void 0 ? _a : core_1.utils.uuid(),
        });
        await this.openId4VcVerifierRepository.save(agentContext, openId4VcVerifier);
        await (0, router_1.storeActorIdForContextCorrelationId)(agentContext, openId4VcVerifier.verifierId);
        return openId4VcVerifier;
    }
    async getRelyingParty(agentContext, verifier, { presentationDefinition, requestSigner, clientId, }) {
        const authorizationResponseUrl = (0, core_1.joinUriParts)(this.config.baseUrl, [
            verifier.verifierId,
            this.config.authorizationEndpoint.endpointPath,
        ]);
        const signatureSuiteRegistry = agentContext.dependencyManager.resolve(core_1.SignatureSuiteRegistry);
        const supportedAlgs = (0, utils_1.getSupportedJwaSignatureAlgorithms)(agentContext);
        const supportedProofTypes = signatureSuiteRegistry.supportedProofTypes;
        // Check: audience must be set to the issuer with dynamic disc otherwise self-issued.me/v2.
        const builder = did_auth_siop_1.RP.builder();
        let _clientId = clientId;
        if (requestSigner) {
            const suppliedSignature = await (0, utils_1.getSphereonSuppliedSignatureFromJwtIssuer)(agentContext, requestSigner);
            builder.withSignature(suppliedSignature);
            _clientId = suppliedSignature.did;
        }
        if (!_clientId) {
            throw new core_1.CredoError("Either 'requestSigner' or 'clientId' must be provided.");
        }
        // FIXME: we now manually remove did:peer, we should probably allow the user to configure this
        const supportedDidMethods = agentContext.dependencyManager
            .resolve(core_1.DidsApi)
            .supportedResolverMethods.filter((m) => m !== 'peer');
        builder
            .withRedirectUri(authorizationResponseUrl)
            .withIssuer(did_auth_siop_1.ResponseIss.SELF_ISSUED_V2)
            .withSupportedVersions([did_auth_siop_1.SupportedVersion.SIOPv2_D11, did_auth_siop_1.SupportedVersion.SIOPv2_D12_OID4VP_D18])
            // TODO: we should probably allow some dynamic values here
            .withClientMetadata({
            client_id: _clientId,
            passBy: did_auth_siop_1.PassBy.VALUE,
            idTokenSigningAlgValuesSupported: supportedAlgs,
            responseTypesSupported: [did_auth_siop_1.ResponseType.VP_TOKEN, did_auth_siop_1.ResponseType.ID_TOKEN],
            subject_syntax_types_supported: supportedDidMethods.map((m) => `did:${m}`),
            vpFormatsSupported: {
                jwt_vc: {
                    alg: supportedAlgs,
                },
                jwt_vc_json: {
                    alg: supportedAlgs,
                },
                jwt_vp: {
                    alg: supportedAlgs,
                },
                ldp_vc: {
                    proof_type: supportedProofTypes,
                },
                ldp_vp: {
                    proof_type: supportedProofTypes,
                },
                'vc+sd-jwt': {
                    kb_jwt_alg_values: supportedAlgs,
                    sd_jwt_alg_values: supportedAlgs,
                },
            },
        })
            .withCustomResolver((0, utils_1.getSphereonDidResolver)(agentContext))
            .withResponseMode(did_auth_siop_1.ResponseMode.POST)
            .withResponseType(presentationDefinition ? [did_auth_siop_1.ResponseType.ID_TOKEN, did_auth_siop_1.ResponseType.VP_TOKEN] : did_auth_siop_1.ResponseType.ID_TOKEN)
            .withScope('openid')
            .withHasher(core_1.Hasher.hash)
            // TODO: support hosting requests within AFJ and passing it by reference
            .withRequestBy(did_auth_siop_1.PassBy.VALUE)
            .withCheckLinkedDomain(did_auth_siop_1.CheckLinkedDomain.NEVER)
            // FIXME: should allow verification of revocation
            // .withRevocationVerificationCallback()
            .withRevocationVerification(did_auth_siop_1.RevocationVerification.NEVER)
            .withSessionManager(this.config.getSessionManager(agentContext))
            .withEventEmitter(this.config.getEventEmitter(agentContext));
        if (presentationDefinition) {
            builder.withPresentationDefinition({ definition: presentationDefinition }, [did_auth_siop_1.PropertyTarget.REQUEST_OBJECT]);
        }
        for (const supportedDidMethod of supportedDidMethods) {
            builder.addDidMethod(supportedDidMethod);
        }
        return builder.build();
    }
    getPresentationVerificationCallback(agentContext, options) {
        return async (encodedPresentation, presentationSubmission) => {
            this.logger.debug(`Presentation response`, core_1.JsonTransformer.toJSON(encodedPresentation));
            this.logger.debug(`Presentation submission`, presentationSubmission);
            if (!encodedPresentation)
                throw new core_1.CredoError('Did not receive a presentation for verification.');
            let isValid;
            // TODO: it might be better here to look at the presentation submission to know
            // If presentation includes a ~, we assume it's an SD-JWT-VC
            if (typeof encodedPresentation === 'string' && encodedPresentation.includes('~')) {
                const sdJwtVcApi = agentContext.dependencyManager.resolve(core_1.SdJwtVcApi);
                const verificationResult = await sdJwtVcApi.verify({
                    compactSdJwtVc: encodedPresentation,
                    keyBinding: {
                        audience: options.audience,
                        nonce: options.nonce,
                    },
                });
                isValid = verificationResult.verification.isValid;
            }
            else if (typeof encodedPresentation === 'string') {
                const verificationResult = await this.w3cCredentialService.verifyPresentation(agentContext, {
                    presentation: encodedPresentation,
                    challenge: options.nonce,
                    domain: options.audience,
                });
                isValid = verificationResult.isValid;
            }
            else {
                const verificationResult = await this.w3cCredentialService.verifyPresentation(agentContext, {
                    presentation: core_1.JsonTransformer.fromJSON(encodedPresentation, core_1.W3cJsonLdVerifiablePresentation),
                    challenge: options.nonce,
                    domain: options.audience,
                });
                isValid = verificationResult.isValid;
            }
            // FIXME: we throw an error here as there's a bug in sphereon library where they
            // don't check the returned 'verified' property and only catch errors thrown.
            // Once https://github.com/Sphereon-Opensource/SIOP-OID4VP/pull/70 is merged we
            // can remove this.
            if (!isValid) {
                throw new core_1.CredoError('Presentation verification failed.');
            }
            return {
                verified: isValid,
            };
        };
    }
};
OpenId4VcSiopVerifierService = __decorate([
    (0, core_1.injectable)(),
    __param(0, (0, core_1.inject)(core_1.InjectionSymbols.Logger)),
    __metadata("design:paramtypes", [Object, core_1.W3cCredentialService,
        repository_1.OpenId4VcVerifierRepository,
        OpenId4VcVerifierModuleConfig_1.OpenId4VcVerifierModuleConfig])
], OpenId4VcSiopVerifierService);
exports.OpenId4VcSiopVerifierService = OpenId4VcSiopVerifierService;
//# sourceMappingURL=OpenId4VcSiopVerifierService.js.map